# Copyright 2020 The pybadge Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"Tests for app"

import pytest

import app


@pytest.fixture
def client():
    with app.app.test_client() as client:
        yield client


def test_image(client):
    rv = client.get("/img?left_text=build&right_text=passing")
    assert b'build' in rv.data
    assert b'passing' in rv.data



# Copyright 2018 The pybadge Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Tests for PrecalculatedTextMeasurer."""

import unittest

from pybadges import precalculated_text_measurer


class TestPrecalculatedTextMeasurer(unittest.TestCase):

    def test_some_known_widths(self):
        measurer = precalculated_text_measurer.PrecalculatedTextMeasurer(
            default_character_width=5.1,
            char_to_width={
                'H': 1.2,
                'l': 1.3
            },
            pair_to_kern={})

        text_width = measurer.text_width('Hello')
        self.assertAlmostEqual(text_width, 1.2 + 5.1 + 1.3 + 1.3 + 5.1)

    def test_kern_in_middle(self):
        measurer = precalculated_text_measurer.PrecalculatedTextMeasurer(
            default_character_width=5,
            char_to_width={},
            pair_to_kern={
                'el': 3.3,
                'll': 4.4,
                'no': 5.5
            })

        text_width = measurer.text_width('Hello')
        self.assertAlmostEqual(text_width, 5 * 5 - 3.3 - 4.4)

    def test_kern_at_start(self):
        measurer = precalculated_text_measurer.PrecalculatedTextMeasurer(
            default_character_width=5,
            char_to_width={},
            pair_to_kern={
                'He': 3.3,
                'no': 4.4
            })

        text_width = measurer.text_width('Hello')
        self.assertAlmostEqual(text_width, 5 * 5 - 3.3)

    def test_kern_at_end(self):
        measurer = precalculated_text_measurer.PrecalculatedTextMeasurer(
            default_character_width=5,
            char_to_width={},
            pair_to_kern={
                'lo': 3.3,
                'no': 4.4
            })

        text_width = measurer.text_width('Hello')
        self.assertAlmostEqual(text_width, 5 * 5 - 3.3)

    def test_default_usable(self):
        measurer = (
            precalculated_text_measurer.PrecalculatedTextMeasurer.default())
        measurer.text_width('This is a long string of text')


if __name__ == '__main__':
    unittest.main()



# Copyright 2018 The pybadge Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Creates a github-style badge as a SVG image.

This package seeks to generate semantically-identical output to the JavaScript
gh-badges library
(https://github.com/badges/shields/blob/master/doc/gh-badges.md)

>>> badge(left_text='coverage', right_text='23%', right_color='red')
'<svg...</svg>'
>>> badge(left_text='build', right_text='green', right_color='green',
...       whole_link="http://www.example.com/")
'<svg...</svg>'
>>> # base64-encoded PNG image
>>> image_data = 'iVBORw0KGgoAAAANSUhEUgAAAAIAAAACCAIAAAD91JpzAAAAD0lEQVQI12P4zwAD/xkYAA/+Af8iHnLUAAAAAElFTkSuQmCC'
>>> badge(left_text='build', right_text='green', right_color='green',
...       logo="data:image/png;base64," + image_data)
'<svg...</svg>'
"""

import base64
import imghdr
import mimetypes
from typing import Optional
import urllib.parse
from xml.dom import minidom

import jinja2
import requests

from pybadges import text_measurer
from pybadges import precalculated_text_measurer
from pybadges.version import __version__

_JINJA2_ENVIRONMENT = jinja2.Environment(
    trim_blocks=True,
    lstrip_blocks=True,
    loader=jinja2.PackageLoader('pybadges', '.'),
    autoescape=jinja2.select_autoescape(['svg']))

# Use the same color scheme as describe in:
# https://github.com/badges/shields/blob/master/lib/colorscheme.json

_NAME_TO_COLOR = {
    'brightgreen': '#4c1',
    'green': '#97CA00',
    'yellow': '#dfb317',
    'yellowgreen': '#a4a61d',
    'orange': '#fe7d37',
    'red': '#e05d44',
    'blue': '#007ec6',
    'grey': '#555',
    'gray': '#555',
    'lightgrey': '#9f9f9f',
    'lightgray': '#9f9f9f',
    'critical': '#e05d44',
    'important': '#fe7d37',
    'success': '#4c1',
    'informational': '#007ec6',
    'inactive': '#9f9f9f',
}


def _remove_blanks(node):
    for x in node.childNodes:
        if x.nodeType == minidom.Node.TEXT_NODE:
            if x.nodeValue:
                x.nodeValue = x.nodeValue.strip()
        elif x.nodeType == minidom.Node.ELEMENT_NODE:
            _remove_blanks(x)


def _embed_image(url: str) -> str:
    parsed_url = urllib.parse.urlparse(url)

    if parsed_url.scheme == 'data':
        return url
    elif parsed_url.scheme.startswith('http'):
        r = requests.get(url)
        r.raise_for_status()
        content_type = r.headers.get('content-type')
        if content_type is None:
            raise ValueError('no "Content-Type" header')
        content_type, image_type = content_type.split('/')
        if content_type != 'image':
            raise ValueError(
                'expected an image, got "{0}"'.format(content_type))
        image_data = r.content
    elif parsed_url.scheme:
        raise ValueError('unsupported scheme "{0}"'.format(parsed_url.scheme))
    else:
        with open(url, 'rb') as f:
            image_data = f.read()
        image_type = imghdr.what(None, image_data)
        if not image_type:
            mime_type, _ = mimetypes.guess_type(url, strict=False)
            if not mime_type:
                raise ValueError('not able to determine file type')
            else:
                content_type, image_type = mime_type.split('/')
                if content_type != 'image':
                    raise ValueError('expected an image, got "{0}"'.format(
                        content_type or 'unknown'))

    encoded_image = base64.b64encode(image_data).decode('ascii')
    return 'data:image/{};base64,{}'.format(image_type, encoded_image)


def badge(
    left_text: str,
    right_text: Optional[str] = None,
    left_link: Optional[str] = None,
    right_link: Optional[str] = None,
    center_link: Optional[str] = None,
    whole_link: Optional[str] = None,
    logo: Optional[str] = None,
    left_color: str = '#555',
    right_color: str = '#007ec6',
    center_color: Optional[str] = None,
    measurer: Optional[text_measurer.TextMeasurer] = None,
    left_title: Optional[str] = None,
    right_title: Optional[str] = None,
    center_title: Optional[str] = None,
    whole_title: Optional[str] = None,
    right_image: Optional[str] = None,
    center_image: Optional[str] = None,
    embed_logo: bool = False,
    embed_right_image: bool = False,
    embed_center_image: bool = False,
    id_suffix: str = '',
) -> str:
    """Creates a github-style badge as an SVG image.

    >>> badge(left_text='coverage', right_text='23%', right_color='red')
    '<svg...</svg>'
    >>> badge(left_text='build', right_text='green', right_color='green',
    ...       whole_link="http://www.example.com/")
    '<svg...</svg>'

    Args:
        left_text: The text that should appear on the left-hand-side of the
            badge e.g. "coverage".
        right_text: The text that should appear on the right-hand-side of the
            badge e.g. "23%".
        left_link: The URL that should be redirected to when the left-hand text
            is selected.
        right_link: The URL that should be redirected to when the right-hand
            text is selected.
        whole_link: The link that should be redirected to when the badge is
            selected. If set then left_link and right_right may not be set.
        logo: A url representing a logo that will be displayed inside the
            badge. Can be a data URL e.g. "data:image/svg+xml;utf8,<svg..."
        left_color: The color of the part of the badge containing the left-hand
            text. Can be an valid CSS color
            (see https://developer.mozilla.org/en-US/docs/Web/CSS/color) or a
            color name defined here:
            https://github.com/badges/shields/blob/master/badge-maker/lib/color.js
        right_color: The color of the part of the badge containing the
            right-hand text. Can be an valid CSS color
            (see https://developer.mozilla.org/en-US/docs/Web/CSS/color) or a
            color name defined here:
            https://github.com/badges/shields/blob/master/badge-maker/lib/color.js
        measurer: A text_measurer.TextMeasurer that can be used to measure the
            width of left_text and right_text.
        embed_logo: If True then embed the logo image directly in the badge.
            This can prevent an HTTP request and some browsers will not render
            external image referenced. When True, `logo` must be a HTTP/HTTPS
            URI or a filesystem path. Also, the `badge` call may raise an
            exception if the logo cannot be loaded, is not an image, etc.
        whole_title: The title attribute to associate with the entire badge.
            See https://developer.mozilla.org/en-US/docs/Web/SVG/Element/title.
        left_title: The title attribute to associate with the left part of the
            badge.
            See https://developer.mozilla.org/en-US/docs/Web/SVG/Element/title.
        right_title: The title attribute to associate with the right part of
            the badge.
            See https://developer.mozilla.org/en-US/docs/Web/SVG/Element/title.
        id_suffix: The suffix of the id attributes used in the SVG's elements.
            Use to prevent duplicate ids if several badges are embedded on the
            same page.
    """
    if measurer is None:
        measurer = (
            precalculated_text_measurer.PrecalculatedTextMeasurer.default())

    if (left_link or right_link or center_link) and whole_link:
        raise ValueError(
            'whole_link may not bet set with left_link, right_link, or center_link'
        )

    if center_image and not (right_image or right_text):
        raise ValueError('cannot have a center_image without a right element')

    if (center_image and not center_color) or (not center_image and
                                               center_color):
        raise ValueError('must have both a center_image and a center_color')

    if logo and embed_logo:
        logo = _embed_image(logo)

    if right_image and embed_right_image:
        right_image = _embed_image(right_image)

    if center_image and embed_center_image:
        center_image = _embed_image(center_image)

    if center_color:
        center_color = _NAME_TO_COLOR.get(center_color, center_color)

    right_text_width = None
    if right_text:
        right_text_width = measurer.text_width(right_text) / 10.0

    template = _JINJA2_ENVIRONMENT.get_template('badge-template-full.svg')

    svg = template.render(
        left_text=left_text,
        right_text=right_text,
        left_text_width=measurer.text_width(left_text) / 10.0,
        right_text_width=right_text_width,
        left_link=left_link,
        right_link=right_link,
        whole_link=whole_link,
        center_link=center_link,
        logo=logo,
        left_color=_NAME_TO_COLOR.get(left_color, left_color),
        right_color=_NAME_TO_COLOR.get(right_color, right_color),
        center_color=center_color,
        left_title=left_title,
        right_title=right_title,
        center_title=center_title,
        whole_title=whole_title,
        right_image=right_image,
        center_image=center_image,
        id_suffix=id_suffix,
    )
    xml = minidom.parseString(svg)
    _remove_blanks(xml)
    xml.normalize()
    return xml.documentElement.toxml()



# Copyright 2018 The pybadge Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Output a github-style badge as an SVG image given some text and colors.

For more information, run:
$ python3 -m pybadges --help
"""

import argparse
import sys
import tempfile
import webbrowser

sys.path.append('/home/nick/git/pybadges/')
import pybadges
from pybadges.version import __version__


def main():
    parser = argparse.ArgumentParser(
        'pybadges',
        description='generate a github-style badge given some text and colors')

    parser.add_argument(
        '--left-text',
        default='license',
        help='the text to show on the left-hand-side of the badge')
    parser.add_argument(
        '--right-text',
        default=None,
        help='the text to show on the right-hand-side of the badge')
    parser.add_argument(
        '--left-link',
        default=None,
        help='the url to redirect to when the left-hand of the badge is ' +
        'clicked')
    parser.add_argument(
        '--right-link',
        default=None,
        help='the url to redirect to when the right-hand of the badge is ' +
        'clicked')
    parser.add_argument(
        '--center-link',
        default=None,
        help='the url to redirect to when the center of the badge is ' +
        'clicked')
    parser.add_argument('--whole-link',
                        default=None,
                        help='the url to redirect to when the badge is clicked')
    parser.add_argument(
        '--logo',
        default=None,
        help='a URI reference to a logo to display in the badge')
    parser.add_argument(
        '--left-color',
        default='#555',
        help='the background color of the left-hand-side of the badge')
    parser.add_argument(
        '--right-color',
        default='#007ec6',
        help='the background color of the right-hand-side of the badge')
    parser.add_argument(
        '--center-color',
        default=None,
        help='the background color of the right-hand-side of the badge')
    parser.add_argument('--browser',
                        action='store_true',
                        default=False,
                        help='display the badge in a browser tab')
    parser.add_argument(
        '--use-pil-text-measurer',
        action='store_true',
        default=False,
        help='use the PilMeasurer to measure the length of text (kerning may '
        'be more precise for non-Western languages. ' +
        '--deja-vu-sans-path must also be set.')
    parser.add_argument(
        '--deja-vu-sans-path',
        default=None,
        help='the path to the ttf font file containing DejaVu Sans. If not ' +
        'present on your system, you can download it from ' +
        'https://www.fontsquirrel.com/fonts/dejavu-sans')
    parser.add_argument(
        '--left-title',
        default=None,
        help='the title to associate with the left part of the badge. See '
        'https://developer.mozilla.org/en-US/docs/Web/SVG/Element/title')
    parser.add_argument(
        '--right-title',
        default=None,
        help='the title to associate with the right part of the badge. See '
        'https://developer.mozilla.org/en-US/docs/Web/SVG/Element/title')
    parser.add_argument(
        '--center-title',
        default=None,
        help='the title to associate with the center part of the badge. See '
        'https://developer.mozilla.org/en-US/docs/Web/SVG/Element/title')
    parser.add_argument(
        '--whole-title',
        default=None,
        help='the title to associate with the entire badge. See '
        'https://developer.mozilla.org/en-US/docs/Web/SVG/Element/title')
    parser.add_argument(
        '--right-image',
        default=None,
        help='the image to associate with the right-hand side of the badge')
    parser.add_argument(
        '--center-image',
        default=None,
        help='the image to associate with the center of the badge')
    parser.add_argument(
        '--embed-logo',
        nargs='?',
        type=lambda x: x.lower() in ['y', 'yes', 't', 'true', '1', ''],
        const='yes',
        default='no',
        help='if the logo is specified then include the image data directly in '
        'the badge (this will prevent a URL fetch and may work around the '
        'fact that some browsers do not fetch external image references); '
        'only works if --logo is a HTTP/HTTPS URI or a file path')
    parser.add_argument(
        '--embed-right-image',
        nargs='?',
        type=lambda x: x.lower() in ['y', 'yes', 't', 'true', '1', ''],
        const='yes',
        default='no',
        help=
        'if the right image is specified then include the image data directly in '
        'the badge (this will prevent a URL fetch and may work around the '
        'fact that some browsers do not fetch external image references); '
        'only works if --logo is a HTTP/HTTPS URI or a file path')
    parser.add_argument(
        '--embed-center-image',
        nargs='?',
        type=lambda x: x.lower() in ['y', 'yes', 't', 'true', '1', ''],
        const='yes',
        default='no',
        help=
        'if the center image is specified then include the image data directly in '
        'the badge (this will prevent a URL fetch and may work around the '
        'fact that some browsers do not fetch external image references); '
        'only works if --logo is a HTTP/HTTPS URI or a file path')
    parser.add_argument(
        '-v',
        '--version',
        action='version',
        version='%(prog)s {version}'.format(version=__version__))
    args = parser.parse_args()

    if (args.left_link or args.right_link or
            args.center_link) and args.whole_link:
        print('argument --whole-link: cannot be set with ' +
              '--left-link, --right-link, or --center_link',
              file=sys.stderr)
        sys.exit(1)

    measurer = None
    if args.use_pil_text_measurer:
        if args.deja_vu_sans_path is None:
            print('argument --use-pil-text-measurer: must also set ' +
                  '--deja-vu-sans-path',
                  file=sys.stderr)
            sys.exit(1)
        from pybadges import pil_text_measurer
        measurer = pil_text_measurer.PilMeasurer(args.deja_vu_sans_path)

    badge = pybadges.badge(left_text=args.left_text,
                           right_text=args.right_text,
                           left_link=args.left_link,
                           right_link=args.right_link,
                           center_link=args.center_link,
                           whole_link=args.whole_link,
                           logo=args.logo,
                           left_color=args.left_color,
                           right_color=args.right_color,
                           center_color=args.center_color,
                           measurer=measurer,
                           left_title=args.left_title,
                           right_title=args.right_title,
                           center_title=args.center_title,
                           whole_title=args.whole_title,
                           right_image=args.right_image,
                           center_image=args.center_image,
                           embed_logo=args.embed_logo,
                           embed_right_image=args.embed_right_image,
                           embed_center_image=args.embed_center_image)

    if args.browser:
        _, badge_path = tempfile.mkstemp(suffix='.svg')
        with open(badge_path, 'w') as f:
            f.write(badge)

        webbrowser.open_new_tab('file://' + badge_path)
    else:
        print(badge, end='')


main()



# Copyright 2018 The pybadge Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Tests for pybadges."""

import base64
import doctest
import json
import os.path
import pathlib
import sys
import tempfile
import unittest
import xmldiff.main

import pybadges
from tests import image_server

TEST_DIR = os.path.dirname(__file__)

PNG_IMAGE_B64 = (
    'iVBORw0KGgoAAAANSUhEUgAAAAIAAAACCAIAAAD91JpzAAAAD0lEQVQI12P4zw'
    'AD/xkYAA/+Af8iHnLUAAAAAElFTkSuQmCC')
PNG_IMAGE = base64.b64decode(PNG_IMAGE_B64)


class TestPybadgesBadge(unittest.TestCase):
    """Tests for pybadges.badge."""

    def setUp(self):
        super().setUp()
        self._image_server = image_server.ImageServer(PNG_IMAGE)
        self._image_server.start_server()

    def tearDown(self):
        super().tearDown()
        self._image_server.stop_server()

    def test_docs(self):
        doctest.testmod(pybadges, optionflags=doctest.ELLIPSIS)

    def test_whole_link_and_left_link(self):
        with self.assertRaises(ValueError):
            pybadges.badge(left_text='foo',
                           right_text='bar',
                           left_link='http://example.com/',
                           whole_link='http://example.com/')

    def test_changes(self):
        with open(os.path.join(TEST_DIR, 'test-badges.json'), 'r') as f:
            examples = json.load(f)

        for example in examples:
            self._image_server.fix_embedded_url_reference(example)
            file_name = example.pop('file_name')
            with self.subTest(example=file_name):
                goldenpath = os.path.join(TEST_DIR, 'golden-images', file_name)

                with open(goldenpath, mode="r", encoding="utf-8") as f:
                    golden_image = f.read()
                pybadge_image = pybadges.badge(**example)

                diff = xmldiff.main.diff_texts(golden_image, pybadge_image)
                if diff:
                    with tempfile.NamedTemporaryFile(mode="w+t",
                                                     encoding="utf-8",
                                                     delete=False,
                                                     suffix=".svg") as actual:
                        actual.write(pybadge_image)

                    with tempfile.NamedTemporaryFile(mode="w+t",
                                                     delete=False,
                                                     suffix=".html") as html:
                        html.write("""
                        <html>
                            <body>
                                <img src="file://%s"><br>
                                <img src="file://%s">
                            <body>
                        </html>""" % (goldenpath, actual.name))
                    self.fail(
                        "images for %s differ:\n%s\nview with:\npython -m webbrowser %s"
                        % (file_name, diff, html.name))


class TestEmbedImage(unittest.TestCase):
    """Tests for pybadges._embed_image."""

    def test_data_url(self):
        url = 'data:image/png;base64,' + PNG_IMAGE_B64
        self.assertEqual(url, pybadges._embed_image(url))

    def test_http_url(self):
        url = 'https://dev.w3.org/SVG/tools/svgweb/samples/svg-files/python.svg'
        self.assertRegex(pybadges._embed_image(url),
                         r'^data:image/svg(\+xml)?;base64,')

    def test_not_image_url(self):
        with self.assertRaisesRegex(ValueError,
                                    'expected an image, got "text"'):
            pybadges._embed_image('http://www.google.com/')

    @unittest.skipIf(sys.platform.startswith("win"), "requires Unix filesystem")
    def test_svg_file_path(self):
        image_path = os.path.abspath(
            os.path.join(TEST_DIR, 'golden-images', 'build-failure.svg'))
        self.assertRegex(pybadges._embed_image(image_path),
                         r'^data:image/svg(\+xml)?;base64,')

    @unittest.skipIf(sys.platform.startswith("win"), "requires Unix filesystem")
    def test_png_file_path(self):
        with tempfile.NamedTemporaryFile() as png:
            png.write(PNG_IMAGE)
            png.flush()
            self.assertEqual(pybadges._embed_image(png.name),
                             'data:image/png;base64,' + PNG_IMAGE_B64)

    @unittest.skipIf(sys.platform.startswith("win"), "requires Unix filesystem")
    def test_unknown_type_file_path(self):
        with tempfile.NamedTemporaryFile() as non_image:
            non_image.write(b'Hello')
            non_image.flush()
            with self.assertRaisesRegex(ValueError,
                                        'not able to determine file type'):
                pybadges._embed_image(non_image.name)

    @unittest.skipIf(sys.platform.startswith("win"), "requires Unix filesystem")
    def test_text_file_path(self):
        with tempfile.NamedTemporaryFile(suffix='.txt') as non_image:
            non_image.write(b'Hello')
            non_image.flush()
            with self.assertRaisesRegex(ValueError,
                                        'expected an image, got "text"'):
                pybadges._embed_image(non_image.name)

    def test_file_url(self):
        image_path = os.path.abspath(
            os.path.join(TEST_DIR, 'golden-images', 'build-failure.svg'))

        with self.assertRaisesRegex(ValueError, 'unsupported scheme "file"'):
            pybadges._embed_image(pathlib.Path(image_path).as_uri())


if __name__ == '__main__':
    unittest.main()



# Copyright 2020 The pybadge Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
""" Example Flask server that serves badges."""

import flask
import pybadges

app = flask.Flask(__name__)


@app.route('/')
@app.route('/index')
def index():
    """Serve an HTML page containing badge images."""
    badges = [
        {
            'left_text': 'Build',
            'right_text': 'passing',
            'left_color': '#555',
            'right_color': '#008000'
        },
        {
            'left_text': 'Build',
            'right_text': 'fail',
            'left_color': '#555',
            'right_color': '#800000'
        },
        {
            "left_text":
                "complete",
            "right_text":
                "example",
            "left_color":
                "green",
            "right_color":
                "yellow",
            "logo":
                "data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAAAIAAAACCAIAAAD91JpzAAAAD0lEQVQI12P4zwAD/xkYAA/+Af8iHnLUAAAAAElFTkSuQmCC"
        },
    ]
    for b in badges:
        b['url'] = flask.url_for('.serve_badge', **b)
    return flask.render_template('index.html', badges=badges)


@app.route('/img')
def serve_badge():
    """Serve a badge image based on the request query string."""
    badge = pybadges.badge(left_text=flask.request.args.get('left_text'),
                           right_text=flask.request.args.get('right_text'),
                           left_color=flask.request.args.get('left_color'),
                           right_color=flask.request.args.get('right_color'),
                           logo=flask.request.args.get('logo'))

    response = flask.make_response(badge)
    response.content_type = 'image/svg+xml'
    return response


if __name__ == '__main__':
    app.run()



# Copyright 2022 The pybadge Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""An HTTP image server that can be used to set logo embedding.

The server will respond to any request with the image data provided in the
constructor.
"""

from http import server
import threading


class ImageServer:

    def __init__(self, image_data):
        self._image_data = image_data

    def start_server(self):
        srv = self

        class Handler(server.BaseHTTPRequestHandler):

            def do_GET(self):
                self.send_response(200)
                self.send_header('Content-Type', 'image/png')
                self.end_headers()
                self.wfile.write(srv._image_data)

        self._httpd = server.HTTPServer(('localhost', 0), Handler)
        self.logo_url = "http://localhost:{0}".format(self._httpd.server_port)

        thread = threading.Thread(target=self._httpd.serve_forever)
        thread.start()

    def fix_embedded_url_reference(self, example):
        if example.get("logo") == "<embedded>":
            example["logo"] = self.logo_url

    def stop_server(self):
        self._httpd.shutdown()



#!/usr/bin/env python3

# Copyright 2018 The pybadge Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import argparse
import json
import os
import os.path
import pkg_resources

import pybadges
from tests import image_server
from tests import test_pybadges


def generate_images(source_json_path, target_directory):
    srv = image_server.ImageServer(test_pybadges.PNG_IMAGE)
    srv.start_server()
    try:
        os.makedirs(target_directory, exist_ok=True)
        with open(source_json_path) as f:
            examples = json.load(f)

        for example in examples:
            srv.fix_embedded_url_reference(example)
            filename = os.path.join(target_directory, example.pop('file_name'))
            with open(filename, 'w', encoding='utf-8') as f:
                f.write(pybadges.badge(**example))
    finally:
        srv.stop_server()


def main():
    parser = argparse.ArgumentParser(
        description='generate a github-style badge given some text and colors')

    parser.add_argument(
        '--source-path',
        default=pkg_resources.resource_filename(__name__,
                                                'tests/test-badges.json'),
        help='the text to show on the left-hand-side of the badge')

    parser.add_argument(
        '--destination-dir',
        default=pkg_resources.resource_filename(__name__,
                                                'tests/golden-images'),
        help='the text to show on the left-hand-side of the badge')
    args = parser.parse_args()
    generate_images(args.source_path, args.destination_dir)


if __name__ == '__main__':
    main()



# Copyright 2018 The pybadge Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""A setup module for pybadges."""

import base64
import re

from setuptools import setup


def get_long_description():
    """Transform README.md into a usable long description.

    Replaces relative references to svg images to absolute https references.
    """

    with open('README.md') as f:
        read_me = f.read()

    def replace_relative_with_absolute(match):
        svg_path = match.group(0)[1:-1]
        return ('(https://github.com/google/pybadges/raw/master/'
                '%s?sanitize=true)' % svg_path)

    return re.sub(r'\(tests/golden-images/.*?\.svg\)',
                  replace_relative_with_absolute, read_me)


setup(
    name='pybadges',
    version='3.0.1',  # Also change in version.py.
    author='Brian Quinlan',
    author_email='brian@sweetapp.com',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Programming Language :: Python :: 3.12',
        'Operating System :: OS Independent',
    ],
    description='A library and command-line tool for generating Github-style ' +
    'badges',
    keywords="github gh-badges badge shield status",
    package_data={
        'pybadges': [
            'badge-template-full.svg', 'default-widths.json', 'py.typed'
        ]
    },
    long_description=get_long_description(),
    long_description_content_type='text/markdown',
    python_requires='>=3.4',
    install_requires=['Jinja2>=3,<4', 'requests>=2.22.0,<3'],
    extras_require={
        'pil-measurement': ['Pillow>=6,<10'],
        'dev': [
            'Flask>=2.0',  # For server tests. 
            'fonttools>=3.26',
            'nox',
            'Pillow>=5',
            'pytest>=3.6',
            'xmldiff>=2.4'
        ],
    },
    license='Apache-2.0',
    packages=["pybadges"],
    url='https://github.com/google/pybadges')



# Copyright 2018 The pybadge Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Nox config for running lint and unit tests."""

import nox
import sys


def _run_tests(session):
    session.run('py.test', '--quiet', 'tests', 'server-example',
                *session.posargs)


@nox.session
def lint(session):
    """Run flake8.
    Returns a failure if flake8 finds linting errors or sufficiently
    serious code quality issues.
    """
    session.install('yapf')
    session.run('yapf', '--diff', '-r', '.')


@nox.session
def unit(session):
    """Run the unit test suite."""
    session.install('-e', '.[dev]')
    session.install('flask')
    _run_tests(session)


@nox.session
@nox.parametrize(
    'install',
    [
        'Jinja2==3.0.0',
        'Pillow==8.3.2',  # Oldest version that supports Python 3.7 to 3.10.
        'requests==2.22.0',
        'xmldiff==2.4'
    ])
def compatibility(session, install):
    """Run the unit test suite with each support library and Python version."""

    session.install(install)
    session.install('-r', 'server-example/requirements-test.txt')
    session.install('-e', '.[dev]')
    _run_tests(session)


@nox.session(python=['3.12'])
def type_check(session):
    """Run type checking using pytype."""
    if sys.platform.startswith('win'):
        session.skip('pytype not supported on Windows')
    session.install('-e', '.[dev]')
    session.install('pytype')
    session.run('pytype', '--python-version=3.7', '--disable=pyi-error',
                'pybadges')
